// Tests a lancer avec:
// java -classpath /usr/share/java/junit4.jar: org.junit.runner.JUnitCore Terrain1DTest

import org.junit.*;
import static org.junit.Assert.*;

public class Terrain1DTest {
    @Test
    public void testTerrain1DVide() {
	Terrain1D t = new Terrain1D(0);
	assertEquals("", t.toString());
	t = new Terrain1D(4);
	assertEquals("    ", t.toString());
    }

    @Test
    public void testTerrain1D() {
	Terrain1D t = new Terrain1D(" . m ");
	assertEquals(" . m ", t.toString());
    }

    @Test
    public void testTerrain1DAleatoire() {
	Terrain1D t = new Terrain1D(5, 0.0);
	assertEquals("     ", t.toString());

	t = new Terrain1D(5, 1.0);
	assertEquals(".....", t.toString());
    }

    @Test
    public void ajouteSouris() {
	Terrain1D t = new Terrain1D(".");
	t.ajouteSouris();
	assertEquals("m", t.toString());

	t = new Terrain1D(". ..   . .");
	t.ajouteSouris();
	assertEquals(1, t.nombreOccurences('m'));
    }

    @Test
    public void ajouteHerbe() {
	Terrain1D t = new Terrain1D(".");
	t.ajouteHerbe();
	assertEquals(".", t.toString());

	t = new Terrain1D("m");
	t.ajouteHerbe();
	assertEquals("m", t.toString());

	t = new Terrain1D("       ");
	t.ajouteHerbe();
	assertEquals(1, t.nombreOccurences('.'));
    }

    @Test
    public void testNombreOccurences() {
	Terrain1D t = new Terrain1D(" ....     m ... m . ");
	assertEquals(0, t.nombreOccurences('x'));
	assertEquals(2, t.nombreOccurences('m'));
	assertEquals(8, t.nombreOccurences('.'));
	assertEquals(10, t.nombreOccurences(' '));

	t = new Terrain1D("");
	assertEquals(0, t.nombreOccurences('x'));
    }

    @Test
    public void testPosition() {
	Terrain1D t = new Terrain1D(". m ....");
	assertEquals(2, t.position('m'));
	t = new Terrain1D(" ....     m ... . ");
	assertEquals(10, t.position('m'));
	t = new Terrain1D(" ....     ... . ");
	assertEquals(-1, t.position('m'));
    }


    @Test
    public void testPlusProchePosition() {
	Terrain1D t = new Terrain1D("..   m ....    . .");
	assertEquals(-1, t.plusProchePosition(0, 'a'));
	assertEquals(0, t.plusProchePosition(0, '.'));
	assertEquals(1, t.plusProchePosition(1, '.'));
	assertEquals(1, t.plusProchePosition(2, '.'));
	assertEquals(1, t.plusProchePosition(2, '.'));
	assertEquals(1, t.plusProchePosition(4, '.'));
	assertEquals(7, t.plusProchePosition(5, '.'));
	assertEquals(5, t.plusProchePosition(15, 'm'));
    }

    @Test
    public void testItere() {
	Terrain1D t = new Terrain1D("..   m ....    . .");
	t.evolue();
	assertEquals("..    m....    . .", t.toString());
	t.evolue();
	assertEquals("..     m...    . .", t.toString());
	t.evolue();
	assertEquals("..      m..    . .", t.toString());
	t.evolue();
	assertEquals("..       m.    . .", t.toString());
	t.evolue();
	assertEquals("..        m    . .", t.toString());
	t.evolue();
	assertEquals("..         m   . .", t.toString());
	t.evolue(); t.evolue(); t.evolue(); t.evolue();
	assertEquals("..             m .", t.toString());
	t.evolue(); t.evolue();
	assertEquals("..               m", t.toString());
	for (int i=0; i<16; i++) t.evolue();
	assertEquals(".m                ", t.toString());
	t.evolue();
	assertEquals("m                 ", t.toString());
	t.evolue();
	assertEquals("                  ", t.toString());
    }

    @Test
    public void testItereHerbePousse() {
	Terrain1D t = new Terrain1D("..    ....    . .");
	for (int i=0; i<20; i++) {
	    t.evolue();
	    assertEquals("..    ....    . .", t.toString());
	}

	t = new Terrain1D("                 ");
	t.setProbaHerbePousse(1);
	t.evolue();
	assertEquals(1, t.nombreOccurences('.'));
    }

    // public static void main(String args[]) {
    // 	org.junit.runner.JUnitCore.main("Terrain1DTest");
    // }
}
